"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isLiteral = exports.assignableTypePairs = exports.typeSymbolName = exports.createRule = void 0;
/* eslint-disable functional/prefer-immutable-types */
const utils_1 = require("@typescript-eslint/utils");
const tsutils_1 = require("tsutils");
exports.createRule = utils_1.ESLintUtils.RuleCreator(
// eslint-disable-next-line functional/functional-parameters
() => "https://github.com/danielnixon/eslint-plugin-total-functions");
const typeSymbolName = (type) => {
    // eslint-disable-next-line functional/no-try-statements
    try {
        // HACK despite what the type suggests, symbol can in fact be undefined
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        return type?.symbol?.name;
    }
    catch {
        // Accessing symbol can throw for reasons I don't fully understand.
        return undefined;
    }
};
exports.typeSymbolName = typeSymbolName;
/**
 * Breaks the supplied types into their union type parts and returns an
 * array of pairs of constituent types that are assignable.
 */
const assignableTypePairs = (rawDestinationType, rawSourceType, checker) => {
    const destinationTypeParts = (0, tsutils_1.unionTypeParts)(rawDestinationType);
    const sourceTypeParts = (0, tsutils_1.unionTypeParts)(rawSourceType);
    return sourceTypeParts.flatMap((sourceTypePart) => destinationTypeParts
        .filter((destinationTypePart) => checker.isTypeAssignableTo(sourceTypePart, destinationTypePart))
        .map((destinationTypePart) => ({
        sourceType: sourceTypePart,
        destinationType: destinationTypePart,
    })));
};
exports.assignableTypePairs = assignableTypePairs;
/**
 * True if this expression is a literal, false otherwise.
 */
const isLiteral = (sourceNode) => {
    // eslint-disable-next-line functional/no-conditional-statements
    if (sourceNode === undefined) {
        return false;
    }
    // eslint-disable-next-line functional/no-conditional-statements
    if (sourceNode.type === utils_1.AST_NODE_TYPES.ObjectExpression) {
        // empty object literal: {}
        return sourceNode.properties.length === 0;
    }
    // eslint-disable-next-line functional/no-conditional-statements
    if (sourceNode.type === utils_1.AST_NODE_TYPES.ArrayExpression) {
        // empty object literal: []
        return sourceNode.elements.length === 0;
    }
    // TODO: handle recursive case for both arrays and objects and
    // permit literals such as string and numbers as properties
    return false;
};
exports.isLiteral = isLiteral;
