"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const no_premature_fp_ts_effects_1 = __importDefault(require("./no-premature-fp-ts-effects"));
const utils_1 = require("@typescript-eslint/utils");
const utils_2 = require("@typescript-eslint/utils");
// eslint-disable-next-line functional/prefer-immutable-types
const ruleTester = new utils_2.ESLintUtils.RuleTester({
    parserOptions: {
        sourceType: "module",
        project: "./tsconfig.tests.json",
    },
    parser: "@typescript-eslint/parser",
});
// eslint-disable-next-line functional/no-expression-statements
ruleTester.run("no-premature-fp-ts-effects", no_premature_fp_ts_effects_1.default, {
    valid: [
        {
            filename: "file.ts",
            code: `
        const myFunc = () => "hello";
        const result: string = myFunc();
      `,
        },
        {
            filename: "file.ts",
            code: `
        const myFunc = (s: string) => "hello " + s;
        const result: string = myFunc("asdf");
      `,
        },
        {
            filename: "file.ts",
            code: `
        export interface Lazy<A> {
          (): A
        }

        const lazy: Lazy<string> = () => "hello";

        const lazyResult: string = lazy();
      `,
        },
        // exercise the try/catch around calleeType.symbol.name (this test prompts it to throw)
        {
            filename: "file.ts",
            code: `
        const foo: { a: () => string } | undefined =
        Date.now() > 0 ? undefined : { a: () => "" };
        foo?.a();
      `,
        },
    ],
    invalid: [
        {
            filename: "file.ts",
            code: `
        export interface IO<A> {
          (): A
        }

        const io: IO<string> = () => "hello";

        const ioResult: string = io();
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.CallExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        export interface Task<A> {
          (): Promise<A>
        }

        const task: Task<string> = () => Promise.resolve("hello");

        const taskResult: Promise<string> = task();
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.CallExpression,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        export interface IO<A> {
          (): A
        }
        const logErrors = (): IO<void> => () => undefined;
        const result = logErrors()();
      `,
            errors: [
                {
                    messageId: "errorStringGeneric",
                    type: utils_1.AST_NODE_TYPES.CallExpression,
                },
            ],
        },
    ],
});
